/*
Copyright 2026 Element Creations Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE in the repository root for full details.
*/

import genWorkflowMermaid from "../../scripts/gen-workflow-mermaid";
import path from "node:path";
import { fileURLToPath } from "node:url";
import { type ResolvedRouteConfig } from "vitepress";

const __dirname = path.dirname(fileURLToPath(import.meta.url));

export default {
    async paths(): Promise<Partial<ResolvedRouteConfig>[]> {
        const root = path.join(__dirname, "..", "..");

        return [
            {
                params: { id: "automations" },
                content: await genWorkflowMermaid([root, path.join(root, "node_modules", "matrix-js-sdk")]),
            },
        ];
    },
};
