/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.subContainer {
    /* This height is from the figma design */
    min-height: 23px;
    width: 100%;
    font: var(--cpd-font-body-md-regular);
    color: var(--cpd-color-text-secondary);
}

.startedTextContainer {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
}
