/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

/** Event tile layout selected by the host surface. */
export type EventLayout = "group" | "bubble" | "irc";

/** Density variant applied within an event layout. */
export type EventDensity = "default" | "compact";
