/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.container {
    min-width: 36px;
    height: 36px;
    border-radius: var(--cpd-space-2x);
    background-color: var(--cpd-color-bg-subtle-secondary);
}

.icon {
    color: var(--cpd-color-icon-primary);
}
