/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

export {
    DEFAULT_EVENT_PRESENTATION,
    EventPresentationProvider,
    useEventPresentation,
    type EventPresentation,
} from "./EventPresentationContext";
export type { EventDensity, EventLayout } from "./EventPresentation.types";
