/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.title {
    font: var(--cpd-font-body-md-semibold);
    color: var(--cpd-color-text-primary);
    /* This height is from the figma design */
    min-height: 23px;
    text-overflow: ellipsis;
    overflow: hidden;
    white-space: nowrap;
    width: 100%;
}

.content {
    flex: 1;
    overflow: hidden;
    /* Width specified to allow the children to overflow and render with ellipsis */
    min-width: 47px;
}

.avatar {
    min-width: 20px;
}

.facepile {
    min-width: fit-content;
}
