/*
Copyright 2024 New Vector Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

// @vitest-environment happy-dom

import { vi, describe, it, expect, beforeEach, afterEach } from "vitest";
import fetchMock from "@fetch-mock/vitest";
import { ClientEvent, MatrixClient } from "matrix-js-sdk/src/matrix";

import { SettingLevel } from "../SettingLevel";
import FallbackIceServerController from "./FallbackIceServerController.ts";
import MatrixClientBackedController from "./MatrixClientBackedController.ts";
import SettingsStore from "../SettingsStore.ts";

describe("FallbackIceServerController", () => {
    beforeEach(() => {
        fetchMock.get("https://matrix.org/_matrix/client/versions", { versions: ["v1.4"] });
    });

    afterEach(() => {
        vi.restoreAllMocks();
    });

    it("should update MatrixClient's state when the setting is updated", async () => {
        const client = new MatrixClient({
            baseUrl: "https://matrix.org",
            userId: "@alice:matrix.org",
            accessToken: "token",
        });
        MatrixClientBackedController.matrixClient = client;

        expect(client.isFallbackICEServerAllowed()).toBeFalsy();
        await SettingsStore.setValue("fallbackICEServerAllowed", null, SettingLevel.DEVICE, true);
        expect(client.isFallbackICEServerAllowed()).toBeTruthy();
    });

    it("should force the setting to be disabled if disable_fallback_ice=true", async () => {
        const controller = new FallbackIceServerController();
        const client = new MatrixClient({
            baseUrl: "https://matrix.org",
            userId: "@alice:matrix.org",
            accessToken: "token",
        });
        MatrixClientBackedController.matrixClient = client;
        expect(controller.settingDisabled).toBeFalsy();

        const wellKnown = {
            "io.element.voip": {
                disable_fallback_ice: true,
            },
        };
        vi.spyOn(client, "getClientWellKnown").mockResolvedValue(wellKnown);
        client.emit(ClientEvent.ClientWellKnown, wellKnown);

        expect(controller.settingDisabled).toBeTruthy();
    });
});
