/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.list {
    position: relative;
    flex: 1;
}

.scrollbar {
    display: flex;
    flex-direction: column;
    flex: 1;
    width: 100%;
}
