/*
Copyright 2025 New Vector Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

import { composeStories } from "@storybook/react-vite";
import { render } from "@test-utils";
import React from "react";
import { describe, it, expect } from "vitest";

import * as stories from "./TextualEventView.stories.tsx";

const { Default, WithLink } = composeStories(stories);

describe("TextualEventView", () => {
    it("renders a textual event", () => {
        const { container } = render(<Default />);
        expect(container).toMatchSnapshot();
    });

    it("renders a textual event with link", () => {
        const { container } = render(<WithLink />);
        expect(container).toMatchSnapshot();
    });
});
